from rest_framework import serializers

from django_resaas.core.base.serializers import BaseSerializer
from django_resaas.models.tipo_entidade import TipoEntidade
from django_resaas.data.group.serializers.grupo import GrupoSerializer

from django_resaas.models.tipo_entidade_group import TipoEntidadeGroup



class TipoEntidadeSerializer(BaseSerializer):
    permanent_fields_files = ['icon']
    estado_display = serializers.CharField(
        source="get_estado_display",
        read_only=True
    )


    groups = serializers.SerializerMethodField()

    def get_groups(self, obj):
        tietgr = []
        for teg in TipoEntidadeGroup.objects.filter(tipo_entidade__id=obj.id):
            tietgr.append({'id':teg.group.id, 'name':teg.group.name})
        # print(tietgr)

        return tietgr

    class Meta:
        model = TipoEntidade
        fields = "__all__"



import uuid

from django.db import models
from django.contrib.contenttypes.models import ContentType


from django_resaas.models.user import User
from django_resaas.models.tipo_entidade import TipoEntidade

from django_resaas.core.base.models import TimeModel


def logo_path(instance, file_name):
    return f'{instance.tipo_entidade.nome}/{instance.nome}/{file_name}'


class Entidade(TimeModel):
    route="view_entidade"
    nome = models.CharField(max_length=100, null=True, default='-')
    site = models.CharField(max_length=300, null=True, default='-')
    logo = models.FileField(upload_to=logo_path, default='logo.png', blank=True)
    display_logo = models.BooleanField(default=True, null=True, blank=True)
    display_bar = models.BooleanField(default=True, null=True, blank=True)
    display_qr = models.BooleanField(default=True, null=True, blank=True)

    tipo_entidade = models.ForeignKey('django_resaas.TipoEntidade', on_delete=models.CASCADE)
    theme = models.ForeignKey('django_resaas.Theme', null=True, blank=True, on_delete=models.SET_NULL)
    layout_settings = models.ForeignKey('django_resaas.LayoutSetting', null=True, blank=True, on_delete=models.SET_NULL)
    typography = models.ForeignKey('django_resaas.Typography', null=True, blank=True, on_delete=models.SET_NULL)
    animation_settings = models.ForeignKey('django_resaas.AnimationSetting', null=True, blank=True, on_delete=models.SET_NULL)

    admins = models.ManyToManyField(User)

    rodape = models.CharField(max_length=2000, null=True)

    disc_space = models.FloatField(default=1048576.0, null=True)
    disc_used_space = models.FloatField(default=0.0, null=True)
    disc_free_space = models.FloatField(default=1048576.0, null=True)

    class Meta:
        permissions = ()

    def save(self, *args, **kwargs):
        if self.disc_free_space is None or self.disc_free_space > self.disc_space:
            self.disc_free_space = self.disc_space - self.disc_used_space
        super().save(*args, **kwargs)

    def __str__(self):
        return self.nome or ''





from rest_framework import serializers


from django_resaas.models.sucursal_user import SucursalUser


class SucursalUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = SucursalUser
        fields = "__all__"




from django.db import models

from django_resaas.core.base.models import TimeModel


class SucursalUserGroup(TimeModel):
    sucursal = models.ForeignKey('django_resaas.Sucursal', on_delete=models.CASCADE)
    user = models.ForeignKey('django_resaas.User', on_delete=models.CASCADE)
    group = models.ForeignKey('auth.Group', on_delete=models.CASCADE)

    class Meta:
        permissions = ()

    def __str__(self):
        return f'{self.user.username} | {self.sucursal.nome} | {self.group.name}'



import json
from datetime import datetime

from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.core.serializers.json import DjangoJSONEncoder

from rest_framework import generics, status
from rest_framework.response import Response
from rest_framework_simplejwt.tokens import RefreshToken

from django_resaas.core.utils.translate import Translate
from django_resaas.core.utils.username import UserName
from django_resaas.core.utils.templates import render_email_template
from django_resaas.core.utils.full_path import FullPath

from django_resaas.models.user import User

from django_resaas.data.user.serializers.register import RegisterSerializer



class RegisterAPIView(generics.GenericAPIView):
    serializer_class = RegisterSerializer

    def post(self, request):
        origin = request.META.get(
            'HTTP_ORIGIN',
            'http://mytech.co.mz'
        )
        nome = origin.split('.')[0].upper().split('/')[-1]

        data = request.data.copy()
        data['username'] = UserName.Create(
            request.data['username'].replace(' ', '_')
        )
        data['mobile'] = None

        serializer = self.serializer_class(data=data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        user = User.objects.get(email=serializer.data['email'])

        logo = FullPath.url(
            None,
            user.tipo_entidade.icon.name
        )

        token = RefreshToken.for_user(user).access_token

        html_message = render_email_template(
            'REGISTER_CONFIRM',
            {
                'username': user.username,
                'token': str(token),
                'entidade': nome,
                'logo': logo,
                'origem': origin,
            }
        )

        mail = EmailMultiAlternatives(
            subject=f'{Translate.tdc(request, "Bem-vindo")} {nome}',
            body='',
            to=[user.email],
        )
        mail.attach_alternative(html_message, 'text/html')

        try:
            mail.send()
        except Exception:
            user.delete()
            return Response(
                {
                    'alert_error': Translate.tdc(
                        request,
                        'Erro ao enviar o email'
                    )
                },
                status=status.HTTP_500_INTERNAL_SERVER_ERROR
            )

        return Response(
            {
                'alert_success': Translate.tdc(
                    request,
                    'Conta criada com sucesso'
                ),
                'data': json.loads(
                    json.dumps(
                        serializer.data,
                        cls=DjangoJSONEncoder
                    )
                ),
            },
            status=status.HTTP_201_CREATED
        )



class DynamicFieldsMixin:
    def get_fields(self):
        fields = super().get_fields()

        include = self.context.get("include_fields")
        exclude = self.context.get("exclude_fields", [])

        if include:
            fields = {k: v for k, v in fields.items() if k in include}

        for field in exclude:
            fields.pop(field, None)

        return fields